<?php
/**
 * Created by PhpStorm.
 * User: robin
 * Date: 7/2/16
 * Time: 10:15 PM
 */

namespace Luticate\Auth\Business;

use Illuminate\Database\Query\Builder;
use Luticate\Auth\DataAccess\LuUserDataAccess;
use Luticate\Auth\Dbo\LuUsersDbo;
use Luticate\Auth\Dbo\LuUsersLiteDbo;
use Luticate\Auth\Dbo\LuUsersLoginDbo;
use Luticate\Auth\Dbo\LuUsersLoginResultDbo;
use Luticate\Utils\Business\LuBusiness;
use Luticate\Utils\Dbo\LuQueryDbo;

class LuUsersBusiness extends LuBusiness
{
    const KEY_USER_ID =  "user_id";
    const KEY_SALT =  "salt";
    const KEY_DATA =  "data";

    /**
     * @return LuUserDataAccess
     */
    protected static function getDataAccess()
    {
        return new LuUserDataAccess();
    }

    protected static function badPassword()
    {
        static::unauthorized("Bad username/password");
    }

    public static function hashPassword($password)
    {
        return password_hash($password, PASSWORD_BCRYPT);
    }

    public static function verifyPassword($password, $hash)
    {
        return password_verify($password, $hash);
    }

    public static function getSalt($length = 10)
    {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    /**
     * @param $user LuUsersDbo
     * @param $data mixed
     * @return string
     */
    public static function getToken($user, $data = null)
    {
        $session_time = 30;//LuticateSettingsBusiness::getValue("LU_SESSION_DAYS");
        return JwtHelper::encode(array(
            self::KEY_USER_ID => $user->getId(),
            self::KEY_SALT => $user->getSalt(),
            self::KEY_DATA => $data
        ), $session_time);
    }

    public static function login(LuUsersLoginDbo $login)
    {
        $user = static::getDataAccess()->getByUsernameOrEmail($login->getUsername());
        if (is_null($user))
            self::badPassword();
        if (!self::verifyPassword($login->getPassword(), $user->getPassword()))
            self::badPassword();

        /**
         * @var $result LuUsersLoginResultDbo
         */
        $result = $user->castAs(LuUsersLoginResultDbo::class);
        $result->setToken(self::getToken($user));
        return $result;
    }

    public static function logout(LuUsersDbo $user)
    {
        if ($user->getId() != 0) {
            $user->setSalt(self::getSalt());
            static::getDataAccess()->editSingleById($user);
        }
        return true;
    }

    /**
     * @param LuQueryDbo $query
     * @return LuUsersLiteDbo[]
     */
    public static function getAllLite(LuQueryDbo $query)
    {
        return static::getDataAccess()->getAll($query)->map(function($user)
        {
            /**
             * @var $user LuUsersDbo
             */
            return $user->castAs(LuUsersLiteDbo::class);
        });
    }
}