<?php
/**
 * Created by PhpStorm.
 * User: robin
 * Date: 7/2/16
 * Time: 10:54 PM
 */

namespace Luticate\Auth\Middleware;

use Luticate\Auth\Business\JwtHelper;
use Luticate\Auth\Business\LuPermissionsBusiness;
use Luticate\Auth\Business\LuUsersBusiness;
use Luticate\Auth\Dbo\LuBuiltInPermissions;
use Luticate\Auth\Dbo\LuUsersDbo;
use Luticate\Utils\Business\LuBusiness;
use Luticate\Utils\Business\LuLog;
use Luticate\Utils\Middleware\LuAbstractMiddleware;

class LuAuthMiddleware implements LuAbstractMiddleware
{
    const TOKEN_HEADER = "X-Luticate-Token";
    
    public function onBefore($_parameters, $_headers, $permissions = [])
    {
        $user = null;
        $token = $_headers[self::TOKEN_HEADER] ?? null;
        if ($token != null) {
            $token = trim($token);
            if ($token == "") {
                $token = null;
            }
        }
        if ($token != null) {
            $data = JwtHelper::decode($token);
            if ($data != null) {
                /**
                 * @var $user LuUsersDbo
                 */
                $user_id = intval($data[LuUsersBusiness::KEY_USER_ID]);
                $salt = $data[LuUsersBusiness::KEY_SALT];
                $user = LuUsersBusiness::getById($user_id);
                if ($user->getSalt() !== $salt) {
                    $user = null;
                }
            }
        }

        if (is_null($user)) {
            if ($token != null) {
                LuBusiness::unauthorized("Invalid token");
            }
            $user = LuUsersBusiness::getById(0);
        }

        $_parameters["_user"] = $user;

        $perm = LuPermissionsBusiness::getUserEffectivePermissionByName($user->getId(), LuBuiltInPermissions::USER_LOGIN);
        if (!$perm) {
            LuBusiness::unauthorized("Account is disabled");
        }
        
        foreach ($permissions as $permission) {
            try {
                $perm = LuPermissionsBusiness::getUserEffectivePermissionByName($user->getId(), $permission);
                if (!$perm) {
                    LuBusiness::unauthorized("Permission denied");
                }
            } catch (\Exception $e)
            {
                LuLog::log($e);
                LuBusiness::unauthorized("Permission denied");
            }
        }

        return $_parameters;
    }
}